import lxml.etree as ET

try:
	FileNotFoundError
except NameError:
	FileNotFoundError = IOError

class ServiceDataEmpty(Exception):
	pass

class ServiceData(object):
	def __init__(self, source):
		self._servicedata = source if isinstance(source, ET._ElementTree) else ET.parse(source)
		self._otp_version = self._servicedata.find("./service[@name='erlang_changes']/param[@name='otp_version']")
		if self._otp_version is None:
			raise ServiceDataEmpty()

	@property
	def otp_version(self):
		return self._otp_version.text

	def set_otp_version(self, otp_version):
		self._otp_version.text = otp_version

	def write(self, sink):
		return self._servicedata.write(sink)

	@staticmethod
	def init_servicedata(source, otp_version):
		try:
			xml = ET.parse(source)
			root = xml.getroot()
		except FileNotFoundError:
			root = ET.fromstring("<servicedata/>")
			xml = ET.ElementTree(root)

		service = root.find("./service[@name='erlang_changes']")
		if service is None:
			service = ET.fromstring("<service name='erlang_changes'/>")
			root.append(service)

		otp_version_node = service.find("./param[@name='otp_version']")
		if otp_version_node is None:
			otp_version_node = ET.fromstring("<param name='otp_version'/>")
			service.append(otp_version_node)

		otp_version_node.text = otp_version

		return ServiceData(xml)



import argparse
import os.path
import glob
import shutil
import erlang_changes
from erlang_changes.changes import Changes
from erlang_changes.service_data import ServiceData, ServiceDataEmpty

try:
	FileNotFoundError
except NameError:
	FileNotFoundError = IOError

parser = argparse.ArgumentParser(prog='otp-service_erlang_changes')
parser.add_argument('--otp_sources', metavar='FILENAME', help='Path to OTP/Erlang sources tarball', required=True)
parser.add_argument('--outdir', help='osc service parameter for internal use only', required=True)

def execute_from_commandline(argv=None):
	args = parser.parse_args(argv)

	otp_src_filename = glob.glob(args.otp_sources)[0]
	otp_src = erlang_changes.OTPSrc.from_file(otp_src_filename)

	otp_version = otp_src.otp_version

	servicedata_filename = "_servicedata"
	tmp_servicedata_filename = os.path.join(args.outdir, "_servicedata")

	try:
		servicedata = ServiceData(servicedata_filename)
	except (ServiceDataEmpty, FileNotFoundError):
		servicedata = ServiceData.init_servicedata(servicedata_filename, otp_version)
		servicedata.write(servicedata_filename)

	prev_otp_version = servicedata.otp_version

	if prev_otp_version == otp_version:
		return

	new_changes = Changes.from_otp_src(otp_src, prev_otp_version)
	for path, changes in Changes.find_changes():
		tmp_changes = os.path.join(args.outdir, changes)
		shutil.copyfile(os.path.join(path, changes), tmp_changes)
		new_changes.write(tmp_changes)

	servicedata.set_otp_version(otp_version)
	servicedata.write(tmp_servicedata_filename)

	shutil.move(tmp_servicedata_filename, servicedata_filename)

	for path, changes in Changes.find_changes():
		tmp_changes = os.path.join(args.outdir, changes)
		shutil.move(tmp_changes, os.path.join(path, changes))

if __name__ == "__main__":
	import sys

	execute_from_commandline(sys.argv[1:])



from erlang_changes.service_data import ServiceData, ServiceDataEmpty
import io
import tempfile
import unittest
import xml.etree.ElementTree as ET

class TestServiceData(unittest.TestCase):
	def assertXMLEqual(self, first, second):
		return self.assertEqual(ET.tostring(first.getroot()), ET.tostring(second.getroot()))

	def test_parse1(self):
		s = u"""<servicedata><service name="erlang_changes"><param name="otp_version">17.0</param></service></servicedata>"""

		c = ServiceData(io.StringIO(s))

		self.assertEqual(c.otp_version, "17.0")

	def test_parse2(self):
		s = u"""<servicedata/>"""

		with self.assertRaises(ServiceDataEmpty):
			c = ServiceData(io.StringIO(s))

	def test_parse3(self):
		s = u"""<servicedata/>"""

		c = ServiceData.init_servicedata(io.StringIO(s), "21.0")

		self.assertEqual(c.otp_version, "21.0")

	def test_parse4(self):
		s = u"""<servicedata><service name="erlang_changes"/></servicedata>"""

		c = ServiceData.init_servicedata(io.StringIO(s), "21.0")

		self.assertEqual(c.otp_version, "21.0")

	def test_parse5(self):
		s = u"""<servicedata><service name="erlang_changes"><param name="otp_version"/></service></servicedata>"""

		c = ServiceData.init_servicedata(io.StringIO(s), "21.0")

		self.assertEqual(c.otp_version, "21.0")

	def test_write1(self):
		s = u"""<servicedata><service name="erlang_changes"><param name="otp_version">17.0</param></service></servicedata>"""
		expected = u"""<servicedata><service name="erlang_changes"><param name="otp_version">21.0</param></service></servicedata>"""

		c = ServiceData(io.StringIO(s))

		c.set_otp_version("21.0")
		self.assertEqual(c.otp_version, "21.0")

		o = io.BytesIO()
		c.write(o)
		self.assertXMLEqual(
			ET.parse(io.BytesIO(o.getvalue())),
			ET.parse(io.StringIO(expected))
		)

if __name__ == '__main__':
	unittest.main()



from setuptools import setup

setup(name='obs-service-erlang_changes',
	version='0.1.0',
	description='Erlang changelog formatting OBS service',
	url='http://github.com/matwey/obs-service-erlang_changes',
	author='Matwey V. Kornilov',
	author_email='matwey.kornilov@gmail.com',
	license='GPL-2.0',
	packages=['erlang_changes'],
	test_suite='tests',
	data_files=[
		('/usr/lib/obs/service', [
			'service/erlang_changes',
			'service/erlang_changes.service']),
	],
	zip_safe=False)
